#コミック
class Comic < Peta::Binder
  load_manifest
  has_many :comic_stories
  belongs_to :author
  
  validates :title, :presence => true, :length => {:maximum => 100}
  validates :visible, :presence => true, :numericality => true, :inclusion => {:in => 0..1}
  validates :author_id, :presence => true, :numericality => true, :existence => {:both => false}
  
  scope :find_index, -> do
    where(arel_table[:visible].gt 0)
  end
  
  scope :find_private, -> (operators) do 
    where(author_id: operators.author.id)
  end
  
  scope :find_by_author, -> (author_id) do 
    find_index.where(author_id: author_id)
  end
  
  scope :with_stories, -> do
    includes(comic_stories: :story)
  end
  
  scope :find_by_story, -> (story_id) do 
    with_stories.find_index.where(Story.arel_table[:id].eq story_id).references(:story)
  end
  
  # scope of find_play
  def self.find_play(id)
    ComicStory.find_play(id)
  end
  
  # scope of find_private_play
  def self.find_private_play(id, operators)
    ComicStory.find_private_play(id, operators)
  end
  
  def supply_default
    self.visible = 0 if self.visible.blank?
    self.author_id = nil
  end
  
  def overwrite operators
    return false unless operators.author
    self.author_id = operators.author.id
  end
  
  def visible? operators
    case super
    when nil # super return
      return true
    when false
      return false
    else
      self.visible > 0
    end
  end
  
  def symbol_filename
  end
  
  def self.public_list_where list
    'comics.visible > 0'
  end
  
  def self.show_opt
    {:include => {:comic_stories => {:story => {}}, :author => {}}}
  end
  
  def scenario
    panels.map {|panel|
      panel.scenario
    }.join
  end
  
  def plain_scenario
    panels.map {|panel|
      panel.plain_scenario
    }.join
  end
  
end
