module Locmare
  module ListGroupModule
    module LibModule
      class PageStatus
        
        class Default
          attr :total, :options
          
          def initialize list, total, options
            @list = list
            @total = total
            @options = options
          end
          
          def to_hash
            {
              :type => :default, :total => @total, :total_page => self.total_page, 
              :page => self.page, :page_size => self.limit, :item_name => @list.item_name
            }
          end
          
          def pager
            @pager ||= ::Locmare::ListGroupModule::LibModule::Pager::Default.new(self, @options)
          end
          
          def offset
            return @offset if @offset
            @offset = (self.page - 1) * self.limit
          end
          
          def limit
            return @limit if @limit
            @limit = PageStatus.page_size(@options) || self.default_page_size
            if @list.limited?
              @limit = self.default_page_size if @limit < 0
              @limit = self.max_page_size if @limit > self.max_page_size
            else  # unlimited api
              @limit = -1 if @limit < 0
            end
            @limit = @total if (@total > 0) and (@limit > @total)
            @limit
          end
          
          def page
            return @page if @page
            @page = PageStatus.page(@options).to_i
            @page = 1 if @page < 1
            @page
          end
          
          def default_page_size
            @list.default_page_size
          end
          
          def max_page_size
            @list.max_page_size
          end
          
          def mod
            @total % self.limit
          end
          
          def adjust
            self.mod == 0 ? 0 : 1
          end
          
          def total_page
            (@total / self.limit) + self.adjust
          end
          
          def pageable?
            self.total_page > 1
          end
          
        end
        
        class Offset
          attr :total, :options
          
          def initialize list, total, options
            @list = list
            @total = total
            @options = options
          end
          
          def to_hash
            {
              :type => :offset, :total => @total, :total_page => self.total_page, 
              :offset => self.offset, :count => self.limit
            }
          end
          
          def pager
            @pager ||= ::Locmare::ListGroupModule::LibModule::Pager::Offset.new(self, @options)
          end
          
          def offset
            return @offset if @offset
            @offset = PageStatus.offset(@options).to_i
            @offset = @total - 1 if @offset >= @total
            @offset = @total - @offset.abs if @offset < 0
            @offset = 0 if @offset < 0
            @offset
          end
          
          def limit
            return @limit if @limit
            @limit = self.count || self.default_page_size
            @limit = self.default_page_size if @limit == 0
            if @list.limited?
              @limit = self.default_page_size if @limit < 0
              @limit = self.max_page_size if @limit > self.max_page_size
            else  # unlimited api
              @limit = -1 if @limit < 0
            end
            @limit = @total if (@total > 0) and (@limit > @total)
            @limit
          end
          
          def count
            @count ||= PageStatus.count(@options)
          end
          
          def first_mod
            self.offset % self.limit
          end
          
          def last_mod
            (@total - self.offset) % self.limit
          end
          
          def first_adjust
            self.first_mod == 0 ? 0 : 1
          end
          
          def last_adjust
            self.last_mod == 0 ? 0 : 1
          end
          
          def page
            return 1 if self.limit < 0
            self.offset / self.limit + 1 + self.first_adjust
          end
          
          def default_page_size
            @list.default_page_size
          end
          
          def max_page_size
            @list.max_page_size
          end
          
          def total_page
            return 1 if self.limit < 0
            @total / self.limit + self.first_adjust + self.last_adjust
          end
          
          def pageable?
            self.total_page > 1
          end
          
        end
        
        class Unlimited
          attr :total, :options
          
          def initialize list, total, options
            @list = list
            @total = total
            @options = options
          end
          
          def to_hash
            {
              :type => :unlimited
            }
          end
          
          def pager
            nil
          end
          
          def offset
            0
          end
          
          def limit
            -1
          end
          
          def pageable?
            false
          end
          
        end
        
        def self.load list, total, options
          #return nil if total == 0
          ps = if self.offset(options) or self.count(options)
            Offset.new list, total, options
          else
            Default.new list, total, options
          end
          ps = Unlimited.new(list, total, options) if ps.limit < 0
          ps
        end
        
        def self.page options
          n = options['page'] || options[:page]
          n ? n.to_i : nil
        end
        
        def self.page_size options
          n = options['page_size'] || options[:page_size]
          n ? n.to_i : nil
        end
        
        def self.offset options
          n = options['offset'] || options[:offset]
          n ? n.to_i : nil
        end
        
        def self.count options
          n = options['count'] || options[:count]
          n ? n.to_i : nil
        end
        
      end
      
    end
  end
end
