# -*- encoding: utf-8 -*-
require 'spec_helper'
#スト紙
describe StorySheetsController do
  before do
    @admin = FactoryGirl.create :admin
    @sp = FactoryGirl.create :system_picture
    @lg = FactoryGirl.create :license_group
    @license = FactoryGirl.create :license, :license_group_id => @lg.id, :system_picture_id => @sp.id
    @user = FactoryGirl.create :user_yas
    @author = FactoryGirl.create :author, :user_id => @user.id
    @comic = FactoryGirl.create :comic, :author_id => @user.author.id
    @story = FactoryGirl.create :story, :comic_id => @comic.id
    @sheet = FactoryGirl.create :sheet, :author_id => @user.author.id
  end
  
if MagicNumber['run_mode'] == 1
  describe '一覧表示に於いて' do
    before do
      sign_in @user
      @story_sheet = FactoryGirl.create :story_sheet, :t => 0, :story_id => @story.id, :sheet_id => @sheet.id, :author_id => @author.id
      StorySheet.stub(:list).and_return([@story_sheet, @story_sheet, @story_sheet])
    end
    context 'パラメータpageについて' do
      it '@pageに値が入る' do
        get :index, :page => 5
        assigns(:page).should eq 5
      end
      it '省略されると@pageに1値が入る' do
        get :index
        assigns(:page).should eq 1
      end
      it '与えられたpage_sizeがセットされている' do
        get :index, :page_size => 15
        assigns(:page_size).should eq 15
      end
      it '省略されると@page_sizeにデフォルト値が入る' do
        get :index
        assigns(:page_size).should eq StorySheet.default_page_size
      end
      it '最大を超えると@page_sizeにデフォルト最大値が入る' do
        get :index, :page_size => 1500
        assigns(:page_size).should eq StorySheet.max_page_size
      end
      it '不正な値が入ると@page_sizeにデフォルト最大値が入る' do
        get :index, :page_size => 0
        assigns(:page_size).should eq StorySheet.default_page_size
      end
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :index
        response.should be_success 
      end
      it 'スト紙モデルに一覧を問い合わせている' do
        StorySheet.should_receive(:list).exactly(1)
        get :index
      end
      it '@story_sheetsにリストを取得している' do
        get :index
        assigns(:story_sheets).should have_at_least(3).items
      end
      context 'html形式' do
        it '@paginateにページ制御を取得している' do
          get :index
          assigns(:paginate).is_a?(Kaminari::PaginatableArray).should be_true
        end
        it 'indexテンプレートを描画する' do
          get :index
          response.should render_template("index")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :index, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it 'スト紙モデルにjson一覧出力オプションを問い合わせている' do
          StorySheet.should_receive(:list_json_opt).exactly(1)
          get :index, :format => :json
        end
        it 'データがリスト構造になっている' do
          get :index, :format => :json
          json = JSON.parse response.body
          json.should have_at_least(3).items
        end
        it 'リストの先頭くらいはスト紙っぽいものであって欲しい' do
          get :index, :format => :json
          json = JSON.parse response.body
          json.first.has_key?("story_id").should be_true
          json.first.has_key?("sheet_id").should be_true
          json.first.has_key?("t").should be_true
        end
      end
    end
    context 'ユーザ権限がないとき' do
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :index
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          get :index
          response.should redirect_to '/users/sign_in'
        end
      end
      context 'json形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          get :index, :format => :json
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          get :index, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
    context 'ユーザ権限はないが管理者権限があるとき' do
      before do
        sign_out @user
        sign_in @admin
      end
      it 'ステータスコード200 OKを返す' do
        get :index
        response.should be_success 
      end
    end
    context 'ユーザだが作家登録していないとき' do
      before do
        @author.destroy
      end
      it 'ステータスコード200 OKを返す' do
        get :index
        response.should be_success 
      end
    end
  end
  
  describe '単体表示に於いて' do
    before do
      sign_in @user
      @story_sheet = FactoryGirl.create :story_sheet, :t => 0, :story_id => @story.id, :sheet_id => @sheet.id, :author_id => @author.id
      Sheet.stub(:show).with(@sheet.id.to_s, [@user, nil]).and_return(@sheet)
      Sheet.stub(:show).with(@sheet.id.to_s, [nil, @admin]).and_return(@sheet)
      StorySheet.stub(:show).with(@story_sheet.id.to_s, [@user, nil]).and_return(@story_sheet)
      StorySheet.stub(:show).with(@story_sheet.id.to_s, [nil, @admin]).and_return(@story_sheet)
    end
    context 'つつがなく終わるとき' do
      it 'スト紙モデルに単体取得を問い合わせている' do
        StorySheet.should_receive(:show).with(@story_sheet.id.to_s, [@user, nil]).exactly(1)
        get :show, :id => @story_sheet.id
      end
      it '@story_sheetにアレを取得している' do
        get :show, :id => @story_sheet.id
        assigns(:story_sheet).should eq @story_sheet
      end
      context 'html形式' do
        it 'ステータスコード200 OKを返す' do
          get :show, :id => @story_sheet.id
          response.should be_success
        end
        it 'showテンプレートを描画する' do
          get :show, :id => @story_sheet.id
          response.should render_template("show")
        end
      end
      context 'json形式' do
        it 'ステータスコード200 OKを返す' do
          get :show, :id => @story_sheet.id, :format => :json
          response.should be_success
        end
        it 'jsonデータを返す' do
          get :show, :id => @story_sheet.id, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it 'スト紙モデルにjson単体出力オプションを問い合わせている' do
          StorySheet.should_receive(:show_json_opt).exactly(1)
          get :show, :id => @story_sheet.id, :format => :json
        end
        it 'データがアレになっている' do
          get :show, :id => @story_sheet.id, :format => :json
          json = JSON.parse response.body
          json.has_key?("story_id").should be_true
          json.has_key?("sheet_id").should be_true
          json.has_key?("author_id").should be_true
        end
      end
    end
    context 'ユーザ権限がないとき' do
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :show, :id => @story_sheet.id
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          get :show, :id => @story_sheet.id
          response.body.should redirect_to '/users/sign_in'
        end
      end
      context 'json形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          get :show, :id => @story_sheet.id, :format => :json
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          get :show, :id => @story_sheet.id, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
    context 'ユーザ権限はないが管理者権限があるとき' do
      before do
        sign_out @user
        sign_in @admin
      end
      it 'ステータスコード200 OKを返す' do
          get :show, :id => @story_sheet.id
        response.should be_success 
      end
    end
    context 'ユーザだが作家登録していないとき' do
      before do
        @author.destroy
      end
      it 'ステータスコード200 OKを返す' do
          get :show, :id => @story_sheet.id
        response.should be_success 
      end
    end
  end
  
  describe '新規作成フォーム表示に於いて' do
    before do
      sign_in @user
    end
    context 'つつがなく終わるとき' do
      it '@story_sheetに新規データを用意している' do
        get :new
        assigns(:story_sheet).should be_a_new(StorySheet)
      end
      it 'モデルにデフォルト値補充を依頼している' do
        StorySheet.any_instance.should_receive(:supply_default).exactly(1)
        get :new
      end
      context 'html形式' do
        it 'ステータスコード200 OKを返す' do
          get :new
          response.should be_success 
        end
        it 'newテンプレートを描画する' do
          get :new
          response.should render_template("new")
        end
      end
      context 'js形式' do
        it 'ステータスコード200 OKを返す' do
          get :new, :format => :js
          response.should be_success 
        end
        it 'new.jsテンプレートを描画する' do
          get :new, :format => :js
          response.should render_template("new")
        end
      end
      context 'json形式' do
        it 'ステータスコード200 OKを返す' do
          get :new, :format => :json
          response.should be_success 
        end
        it 'jsonデータを返す' do
          get :new, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it 'スト紙モデルにjson単体出力オプションを問い合わせている' do
          StorySheet.should_receive(:show_json_opt).exactly(1)
          get :new, :format => :json
        end
        it 'データがアレになっている' do
          get :new, :format => :json
          json = JSON.parse response.body
          json.has_key?("story_id").should be_true
          json.has_key?("sheet_id").should be_true
          json.has_key?("author_id").should be_true
        end
      end
    end
    context 'ユーザ権限がないとき' do
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :new
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          get :new
          response.body.should redirect_to '/users/sign_in'
        end
      end
      context 'json形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          get :new, :format => :json
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          get :new, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
    context 'ユーザ権限はないが管理者権限があるとき' do
      before do
        sign_out @user
        sign_in @admin
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :new
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          get :new
          response.body.should redirect_to '/users/sign_in'
        end
      end
    end
    context 'ユーザだが作家登録していないとき' do
      before do
        @author.destroy
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :new
          response.status.should eq 302
        end
        it '作家登録ページへ遷移する' do
          get :new
          response.body.should redirect_to new_author_path
        end
      end
    end
  end
  
  describe '新規作成に於いて' do
    before do
      @attr = FactoryGirl.attributes_for(:story_sheet, :t => nil, :story_id => @story.id, :sheet_id => @sheet.id, :author_id => @author.id)
      sign_in @user
    end
    context 'つつがなく終わるとき' do
      it 'デフォルト値補充を依頼する' do
        StorySheet.any_instance.should_receive(:supply_default).exactly(1)
        post :create, :story_sheet => @attr
      end
      it 'POSTデータから、カラム値を復元している' do
        StorySheet.any_instance.stub(:store).and_return(true)
        post :create, :story_sheet => @attr
        assigns(:story_sheet).story_id.should eq @story.id
        assigns(:story_sheet).sheet_id.should eq @sheet.id
      end
      it '上書き補充を依頼する' do
        StorySheet.any_instance.should_receive(:overwrite).exactly(1)
        post :create, :story_sheet => @attr
      end
      it 'ストーリーモデルに編集取得を依頼している' do
        Story.stub(:edit).and_return(@story)
        Story.should_receive(:edit).with(@story.id, @author).exactly(1)
        post :create, :story_sheet => @attr
      end
      it '用紙モデルに単体取得を依頼している' do
        Sheet.should_receive(:show).with(@sheet.id, @author).exactly(1)
        post :create, :story_sheet => @attr
      end
      it 'モデルに保存依頼する' do
        StorySheet.any_instance.should_receive(:store).exactly(1)
        post :create, :story_sheet => @attr
      end
      it "@story_sheetに作成されたストーリーを保持していて、それがDBにある" do
        post :create, :story_sheet => @attr
        assigns(:story_sheet).should be_a(StorySheet)
        assigns(:story_sheet).should be_persisted
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          StorySheet.any_instance.stub(:store).and_return(true)
          post :create, :story_sheet => @attr
          response.status.should eq 302
        end
        it 'ストーリーのスト紙表示へ遷移する' do
#          StorySheet.any_instance.stub(:store).and_return(true)
          post :create, :story_sheet => @attr
          response.should redirect_to(play_story_path(@attr[:story_id]))
        end
      end
      context 'json形式' do
        it 'ステータスコード200 OKを返す' do
#          StorySheet.any_instance.stub(:store).and_return(true)
          post :create, :story_sheet => @attr, :format => :json
          response.should be_success 
        end
        it '作成されたストーリーをjsonデータで返す' do
          post :create, :story_sheet => @attr, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it 'スト紙モデルにjson単体出力オプションを問い合わせている' do
          StorySheet.should_receive(:show_json_opt).exactly(1)
          post :create, :story_sheet => @attr, :format => :json
        end
        it 'データがアレになっている' do
          post :create, :story_sheet => @attr, :format => :json
          json = JSON.parse response.body
          json["t"].should eq 0
        end
      end
    end
    context 'ユーザ権限がないとき' do
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          post :create, :story_sheet => @attr
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          post :create, :story_sheet => @attr
          response.body.should redirect_to '/users/sign_in'
        end
      end
      context 'json形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          post :create, :story_sheet => @attr, :format => :json
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          post :create, :story_sheet => @attr, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
    context 'ユーザ権限はないが管理者権限があるとき' do
      before do
        sign_out @user
        sign_in @admin
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          post :create, :story_sheet => @attr
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          post :create, :story_sheet => @attr
          response.body.should redirect_to '/users/sign_in'
        end
      end
    end
    context 'ユーザだが作家登録していないとき' do
      before do
        @author.destroy
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          post :create, :story_sheet => @attr
          response.status.should eq 302
        end
        it '作家登録ページへ遷移する' do
          post :create, :story_sheet => @attr
          response.body.should redirect_to new_author_path
        end
      end
    end
    context '検証、保存に失敗した' do
      before do
        StorySheet.any_instance.stub(:store).and_return(false)
      end
      it "未保存のストーリーを保持している" do
        post :create, :story_sheet => @attr
        assigns(:story_sheet).should be_a_new(StorySheet)
      end
      context 'html形式' do
        it 'ステータスコード200 OKを返す' do
          post :create, :story_sheet => @attr
          response.status.should eq 200
        end
        it '新規ページを描画する' do
          post :create, :story_sheet => @attr
          response.should render_template("new")
        end
      end
      context 'json形式' do
        it 'ステータスコード422 unprocessable_entity を返す' do
          post :create, :story_sheet => @attr, :format => :json
          response.status.should eq 422
        end
        it '応答メッセージUnprocessable Entityを返す' do
          post :create, :story_sheet => @attr, :format => :json
          response.message.should match(/Unprocessable/)
        end
      end
    end
  end

  describe '編集フォーム表示に於いて' do
    before do
      @story_sheet = FactoryGirl.create :story_sheet, :author_id => @author.id
      sign_in @user
      StorySheet.stub(:show).and_return(@story_sheet)
    end
    context 'つつがなく終わるとき' do
      it 'スト紙モデルに編集取得を問い合わせている' do
        StorySheet.should_receive(:edit).exactly(1)
        get :edit, :id => @story_sheet.id
      end
      it '@story_sheetにデータを用意している' do
        get :edit, :id => @story_sheet.id
        assigns(:story_sheet).should eq @story_sheet
      end
      context 'html形式' do
        it 'ステータスコード200 OKを返す' do
          get :edit, :id => @story_sheet.id
          response.should be_success 
        end
        it 'editテンプレートを描画する' do
          get :edit, :id => @story_sheet.id
          response.should render_template("edit")
        end
      end
      context 'js形式' do
        it 'ステータスコード200 OKを返す' do
          get :edit, :id => @story_sheet.id, :format => :js
          response.should be_success 
        end
        it 'edit.jsテンプレートを描画する' do
          get :edit, :id => @story_sheet.id, :format => :js
          response.should render_template("edit")
        end
      end
    end
    context 'ユーザ権限がないとき' do
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :edit, :id => @story_sheet.id
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          get :edit, :id => @story_sheet.id
          response.body.should redirect_to '/users/sign_in'
        end
      end
      context 'js形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          get :edit, :id => @story_sheet.id, :format => :js
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          get :edit, :id => @story_sheet.id, :format => :js
          response.message.should match(/Unauthorized/)
        end
      end
    end
    context 'ユーザ権限はないが管理者権限があるとき' do
      before do
        sign_out @user
        sign_in @admin
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :edit, :id => @story_sheet.id
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          get :edit, :id => @story_sheet.id
          response.body.should redirect_to '/users/sign_in'
        end
      end
    end
    context 'ユーザだが作家登録していないとき' do
      before do
        @author.destroy
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :edit, :id => @story_sheet.id
          response.status.should eq 302
        end
        it '作家登録ページへ遷移する' do
          get :edit, :id => @story_sheet.id
          response.body.should redirect_to new_author_path
        end
      end
    end
  end

  describe '更新に於いて' do
    before do
      @story_sheet = FactoryGirl.create :story_sheet, :author_id => @user.author.id
      @attr = FactoryGirl.attributes_for(:story_sheet, :author_id => @author.id)
      sign_in @user
    end
    context 'つつがなく終わるとき' do
      it 'モデルに編集取得依頼する' do
        StorySheet.stub(:edit).with(any_args).and_return(@story_sheet)
        StorySheet.should_receive(:edit).exactly(1)
        put :update, :id => @story_sheet.id, :story_sheet => @attr
      end
      it 'POSTデータから、カラム値を復元している' do
        StorySheet.any_instance.stub(:store).and_return(true)
        StorySheet.any_instance.should_receive(:attributes=).exactly(1)
        put :update, :id => @story_sheet.id, :story_sheet => @attr
      end
      it '上書き補充を依頼する' do
        StorySheet.any_instance.should_receive(:overwrite).exactly(1)
        put :update, :id => @story_sheet.id, :story_sheet => @attr
      end
      it 'モデルに保存依頼する' do
        StorySheet.any_instance.should_receive(:store).exactly(1)
        put :update, :id => @story_sheet.id, :story_sheet => @attr
      end
      it "@story_sheetに作成されたスト紙を保持していて、それがDBにある" do
        put :update, :id => @story_sheet.id, :story_sheet => @attr
        assigns(:story_sheet).should be_a(StorySheet)
        assigns(:story_sheet).should be_persisted
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          StorySheet.any_instance.stub(:store).and_return(true)
          put :update, :id => @story_sheet.id, :story_sheet => @attr
          response.status.should eq 302
        end
        it 'スト紙表示へ遷移する' do
#          StorySheet.any_instance.stub(:store).and_return(true)
          put :update, :id => @story_sheet.id, :story_sheet => @attr
          response.should redirect_to(play_story_path(@attr[:story_id]))
        end
      end
      context 'json形式' do
        it 'ステータスコード200 OKを返す' do
#          StorySheet.any_instance.stub(:store).and_return(true)
          put :update, :id => @story_sheet.id, :story_sheet => @attr, :format => :json
          response.should be_success 
        end
      end
    end
    context 'ユーザ権限がないとき' do
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          put :update, :id => @story_sheet.id, :story_sheet => @attr
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          put :update, :id => @story_sheet.id, :story_sheet => @attr
          response.body.should redirect_to '/users/sign_in'
        end
      end
      context 'json形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          put :update, :id => @story_sheet.id, :story_sheet => @attr, :format => :json
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          put :update, :id => @story_sheet.id, :story_sheet => @attr, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
    context 'ユーザ権限はないが管理者権限があるとき' do
      before do
        sign_out @user
        sign_in @admin
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          put :update, :id => @story_sheet.id, :story_sheet => @attr
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          put :update, :id => @story_sheet.id, :story_sheet => @attr
          response.body.should redirect_to '/users/sign_in'
        end
      end
    end
    context 'ユーザだが作家登録していないとき' do
      before do
        @author.destroy
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          put :update, :id => @story_sheet.id, :story_sheet => @attr
          response.status.should eq 302
        end
        it '作家登録ページへ遷移する' do
          put :update, :id => @story_sheet.id, :story_sheet => @attr
          response.body.should redirect_to new_author_path
        end
      end
    end
    context '検証、保存に失敗した' do
      before do
        StorySheet.any_instance.stub(:store).and_return(false)
      end
      it "指定のスト紙を保持している" do
        put :update, :id => @story_sheet.id, :story_sheet => @attr
        assigns(:story_sheet).should eq @story_sheet
      end
      context 'html形式' do
        it 'ステータスコード200 OKを返す' do
          put :update, :id => @story_sheet.id, :story_sheet => @attr
          response.status.should eq 200
        end
        it '編集ページを描画する' do
          put :update, :id => @story_sheet.id, :story_sheet => @attr
          response.should render_template("edit")
        end
      end
      context 'json形式' do
        it 'ステータスコード422 unprocessable_entity を返す' do
          put :update, :id => @story_sheet.id, :story_sheet => @attr, :format => :json
          response.status.should eq 422
        end
        it '応答メッセージUnprocessable Entityを返す' do
          put :update, :id => @story_sheet.id, :story_sheet => @attr, :format => :json
          response.message.should match(/Unprocessable/)
        end
      end
    end
  end

  describe '削除に於いて' do
    before do
      @story_sheet = FactoryGirl.create :story_sheet, :author_id => @author.id
      sign_in @user
      StorySheet.stub(:edit).and_return(@story_sheet)
    end
    context '事前チェックしておく' do
      before do
        StorySheet.stub(:edit).with(any_args()).and_return @story_sheet
        StorySheet.any_instance.stub(:destroy_and_shorten).with(any_args()).and_return(true)
      end
      it 'スト紙モデルに編集取得を問い合わせている' do
        StorySheet.should_receive(:edit).exactly(1)
        delete :destroy, :id => @story_sheet.id
      end
      it 'モデルに削除を依頼する' do
        StorySheet.any_instance.should_receive(:destroy_and_shorten).exactly(1)
        delete :destroy, :id => @story_sheet.id
      end
      it '@story_sheetにアレを取得している' do
        delete :destroy, :id => @story_sheet.id
        assigns(:story_sheet).id.should eq(@story_sheet.id)
      end
    end
    context 'つつがなく終わるとき' do
      before do
        StorySheet.any_instance.stub(:destroy_and_shorten).with(any_args()).and_return(true)
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          delete :destroy, :id => @story_sheet.id
          response.status.should eq 302
        end
        it '閲覧ページへ遷移する' do
          delete :destroy, :id => @story_sheet.id
          response.should redirect_to(play_story_path(@story_sheet.story_id))
        end
      end
      context 'json形式' do
        it 'ステータスコード200 OKを返す' do
          delete :destroy, :id => @story_sheet.id, :format => :json
          response.should be_success
        end
      end
    end
    context 'ユーザ権限がないとき' do
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          delete :destroy, :id => @story_sheet.id
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          delete :destroy, :id => @story_sheet.id
          response.body.should redirect_to '/users/sign_in'
        end
      end
      context 'json形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          delete :destroy, :id => @story_sheet.id, :format => :json
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          delete :destroy, :id => @story_sheet.id, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
    context 'ユーザ権限はないが管理者権限があるとき' do
      before do
        sign_out @user
        sign_in @admin
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          delete :destroy, :id => @story_sheet.id
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          delete :destroy, :id => @story_sheet.id
          response.body.should redirect_to '/users/sign_in'
        end
      end
    end
    context 'ユーザだが作家登録していないとき' do
      before do
        @author.destroy
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          delete :destroy, :id => @story_sheet.id
          response.status.should eq 302
        end
        it '作家登録ページへ遷移する' do
          delete :destroy, :id => @story_sheet.id
          response.body.should redirect_to new_author_path
        end
      end
    end
    context '削除に失敗したとき' do
      before do
        StorySheet.any_instance.stub(:destroy_and_shorten).with(any_args()).and_return(false)
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          delete :destroy, :id => @story_sheet.id
          response.status.should eq 302
        end
        it 'その用紙の詳細ページへ遷移する' do
          delete :destroy, :id => @story_sheet.id
          response.should redirect_to(story_sheet_path(@story_sheet))
        end
      end
      context 'json形式' do
        it 'ステータスコード422 unprocessable_entity を返す' do
          delete :destroy, :id => @story_sheet.id, :format => :json
          response.status.should eq 422
        end
        it '応答メッセージUnprocessable Entityを返す' do
          delete :destroy, :id => @story_sheet.id, :format => :json
          response.message.should match(/Unprocessable/)
        end
      end
    end
=begin
    context '対象スト紙がないとき' do
    end
    context '他人のスト紙だったとき' do
    end
=end
  end
  
else
  describe '一覧表示に於いて' do
    before do
      sign_in @user
      @story_sheet = FactoryGirl.create :story_sheet, :t => 0, :sheet_id => @sheet.id, :story_id => @story.id, :author_id => @author.id
      StorySheet.stub(:list).and_return([@story_sheet, @story_sheet, @story_sheet])
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :index
        response.should be_success 
      end
      context 'html形式' do
        it 'indexテンプレートを描画する' do
          get :index
          response.should render_template("index")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :index, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
      end
    end
    context 'ユーザ権限がないとき' do
      before do
        sign_out @user
      end
      it 'ステータスコード200 OKを返す' do
        get :index
        response.should be_success 
      end
      context 'html形式' do
        it 'indexテンプレートを描画する' do
          get :index
          response.should render_template("index")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :index, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
      end
    end
  end
  
  describe '単体表示に於いて' do
    before do
      sign_in @user
      @story_sheet = FactoryGirl.create :story_sheet, :t => 0, :sheet_id => @sheet.id, :story_id => @story.id, :author_id => @author.id
      Sheet.stub(:show).with(@sheet.id.to_s, [nil, nil]).and_return(@sheet)
      Sheet.stub(:show).with(@sheet.id.to_s, [@user, nil]).and_return(@sheet)
      Sheet.stub(:show).with(@sheet.id.to_s, [nil, @admin]).and_return(@sheet)
      StorySheet.stub(:show).with(@story_sheet.id.to_s, [nil, nil]).and_return(@story_sheet)
      StorySheet.stub(:show).with(@story_sheet.id.to_s, [@user, nil]).and_return(@story_sheet)
      StorySheet.stub(:show).with(@story_sheet.id.to_s, [nil, @admin]).and_return(@story_sheet)
    end
    context 'つつがなく終わるとき' do
      context 'html形式' do
        it 'ステータスコード200 OKを返す' do
          get :show, :id => @story_sheet.id
          response.should be_success
        end
        it 'showテンプレートを描画する' do
          get :show, :id => @story_sheet.id
          response.should render_template("show")
        end
      end
      context 'json形式' do
        it 'ステータスコード200 OKを返す' do
          get :show, :id => @story_sheet.id, :format => :json
          response.should be_success
        end
        it 'jsonデータを返す' do
          get :show, :id => @story_sheet.id, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
      end
    end
    context 'ユーザ権限がないとき' do
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード200 OKを返す' do
          get :show, :id => @story_sheet.id
          response.should be_success
        end
        it 'showテンプレートを描画する' do
          get :show, :id => @story_sheet.id
          response.should render_template("show")
        end
      end
      context 'json形式' do
        it 'ステータスコード200 OKを返す' do
          get :show, :id => @story_sheet.id, :format => :json
          response.should be_success
        end
        it 'jsonデータを返す' do
          get :show, :id => @story_sheet.id, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
      end
    end
  end
  
  describe '閲覧に於いて' do
    before do
      @story_sheet = FactoryGirl.create :story_sheet, :t => 0, :sheet_id => @sheet.id, :story_id => @story.id, :author_id => @author.id
      Sheet.stub(:show).with(@sheet.id.to_s, [nil, nil]).and_return(@sheet)
      Sheet.stub(:show).with(@sheet.id.to_s, [@user, nil]).and_return(@sheet)
      Sheet.stub(:show).with(@sheet.id.to_s, [nil, @admin]).and_return(@sheet)
      StorySheet.stub(:count).and_return(10)
      StorySheet.stub(:play_list).with(any_args).and_return([@story_sheet, @story_sheet, @story_sheet])
      sign_in @user
    end
    context 'つつがなく終わるとき' do
      context 'html形式' do
        it 'ステータスコード200 OKを返す' do
          get :sheet, :id => @sheet.id
          response.should be_success 
        end
        it 'sheetテンプレートを描画する' do
          get :sheet, :id => @sheet.id
          response.should render_template("sheet")
        end
      end
      context 'json形式' do
        it 'ステータスコード200 OKを返す' do
          get :sheet, :id => @sheet.id, :format => :json
          response.should be_success 
        end
        it 'jsonデータを返す' do
          get :sheet, :id => @sheet.id, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
      end
    end
    context 'ユーザ権限がないとき' do
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード200 OKを返す' do
          get :sheet, :id => @sheet.id
          response.should be_success 
        end
        it 'sheetテンプレートを描画する' do
          get :sheet, :id => @sheet.id
          response.should render_template("sheet")
        end
      end
      context 'json形式' do
        it 'ステータスコード200 OKを返す' do
          get :sheet, :id => @sheet.id, :format => :json
          response.should be_success 
        end
        it 'jsonデータを返す' do
          get :sheet, :id => @sheet.id, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
      end
    end
  end

  describe '新規作成フォーム表示に於いて' do
    before do
      sign_in @user
    end
    context 'つつがなく終わるとき' do
      context 'html形式' do
        it 'ステータスコード200 OKを返す' do
          get :new
          response.should be_success 
        end
        it 'newテンプレートを描画する' do
          get :new
          response.should render_template("new")
        end
      end
      context 'js形式' do
        it 'ステータスコード200 OKを返す' do
          get :new, :format => :js
          response.should be_success 
        end
        it 'new.jsテンプレートを描画する' do
          get :new, :format => :js
          response.should render_template("new")
        end
      end
      context 'json形式' do
        it 'ステータスコード200 OKを返す' do
          get :new, :format => :json
          response.should be_success 
        end
        it 'jsonデータを返す' do
          get :new, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
      end
    end
    context 'ユーザ権限がないとき' do
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :new
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          get :new
          response.body.should redirect_to '/users/sign_in'
        end
      end
      context 'json形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          get :new, :format => :json
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          get :new, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
  end
  
  describe '新規作成に於いて' do
    before do
      @attr = FactoryGirl.attributes_for(:story_sheet, :t => nil, :sheet_id => @sheet.id, :story_id => @story.id, :author_id => @author.id)
      sign_in @user
    end
    context 'つつがなく終わるとき' do
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          StorySheet.any_instance.stub(:store).and_return(true)
          post :create, :story_sheet => @attr
          response.status.should eq 302
        end
        it '用紙のスト紙表示へ遷移する' do
#          StorySheet.any_instance.stub(:store).and_return(true)
          post :create, :story_sheet => @attr
          response.should redirect_to(:action => :sheet, :id => @attr[:sheet_id])
        end
      end
      context 'json形式' do
        it 'ステータスコード200 OKを返す' do
#          StorySheet.any_instance.stub(:store).and_return(true)
          post :create, :story_sheet => @attr, :format => :json
          response.should be_success 
        end
        it '作成されたストーリーをjsonデータで返す' do
          post :create, :story_sheet => @attr, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
      end
    end
    context 'ユーザ権限がないとき' do
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          post :create, :story_sheet => @attr
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          post :create, :story_sheet => @attr
          response.body.should redirect_to '/users/sign_in'
        end
      end
      context 'json形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          post :create, :story_sheet => @attr, :format => :json
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          post :create, :story_sheet => @attr, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
  end

  describe '編集フォーム表示に於いて' do
    before do
      @story_sheet = FactoryGirl.create :story_sheet, :author_id => @author.id
      sign_in @user
      StorySheet.stub(:show).and_return(@story_sheet)
    end
    context 'つつがなく終わるとき' do
      context 'html形式' do
        it 'ステータスコード200 OKを返す' do
          get :edit, :id => @story_sheet.id
          response.should be_success 
        end
        it 'editテンプレートを描画する' do
          get :edit, :id => @story_sheet.id
          response.should render_template("edit")
        end
      end
      context 'js形式' do
        it 'ステータスコード200 OKを返す' do
          get :edit, :id => @story_sheet.id, :format => :js
          response.should be_success 
        end
        it 'edit.jsテンプレートを描画する' do
          get :edit, :id => @story_sheet.id, :format => :js
          response.should render_template("edit")
        end
      end
    end
    context 'ユーザ権限がないとき' do
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :edit, :id => @story_sheet.id
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          get :edit, :id => @story_sheet.id
          response.body.should redirect_to '/users/sign_in'
        end
      end
      context 'js形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          get :edit, :id => @story_sheet.id, :format => :js
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          get :edit, :id => @story_sheet.id, :format => :js
          response.message.should match(/Unauthorized/)
        end
      end
    end
  end

  describe '更新に於いて' do
    before do
      @story_sheet = FactoryGirl.create :story_sheet, :author_id => @user.author.id
      @attr = FactoryGirl.attributes_for(:story_sheet, :author_id => @author.id)
      sign_in @user
    end
    context 'つつがなく終わるとき' do
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          StorySheet.any_instance.stub(:store).and_return(true)
          put :update, :id => @story_sheet.id, :story_sheet => @attr
          response.status.should eq 302
        end
        it 'スト紙表示へ遷移する' do
#          StorySheet.any_instance.stub(:store).and_return(true)
          put :update, :id => @story_sheet.id, :story_sheet => @attr
          response.should redirect_to(:action => :sheet, :id => @attr[:sheet_id])
        end
      end
      context 'json形式' do
        it 'ステータスコード200 OKを返す' do
#          StorySheet.any_instance.stub(:store).and_return(true)
          put :update, :id => @story_sheet.id, :story_sheet => @attr, :format => :json
          response.should be_success 
        end
      end
    end
    context 'ユーザ権限がないとき' do
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          put :update, :id => @story_sheet.id, :story_sheet => @attr
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          put :update, :id => @story_sheet.id, :story_sheet => @attr
          response.body.should redirect_to '/users/sign_in'
        end
      end
      context 'json形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          put :update, :id => @story_sheet.id, :story_sheet => @attr, :format => :json
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          put :update, :id => @story_sheet.id, :story_sheet => @attr, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
  end

  describe '削除に於いて' do
    before do
      @story_sheet = FactoryGirl.create :story_sheet, :author_id => @author.id
      sign_in @user
      StorySheet.stub(:edit).and_return(@story_sheet)
    end
    context 'つつがなく終わるとき' do
      before do
        StorySheet.any_instance.stub(:destroy_and_shorten).with(any_args()).and_return(true)
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          delete :destroy, :id => @story_sheet.id
          response.status.should eq 302
        end
        it '閲覧ページへ遷移する' do
          delete :destroy, :id => @story_sheet.id
          response.should redirect_to(:controller => 'story_sheets', :action => :sheet, :id => @story_sheet.sheet_id)
        end
      end
      context 'json形式' do
        it 'ステータスコード200 OKを返す' do
          delete :destroy, :id => @story_sheet.id, :format => :json
          response.should be_success
        end
      end
    end
    context 'ユーザ権限がないとき' do
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          delete :destroy, :id => @story_sheet.id
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          delete :destroy, :id => @story_sheet.id
          response.body.should redirect_to '/users/sign_in'
        end
      end
      context 'json形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          delete :destroy, :id => @story_sheet.id, :format => :json
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          delete :destroy, :id => @story_sheet.id, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
  end
  
end
end
